/*********************************************************************
 *                    Flowcode USB Component Code
 *
 * File: pic_usb.c
 *
 * (c) 2009 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 031008 | BR | Created
 * 170409 | BR | Ammended Standard Request function to fix timeout bug in Linux systems
 * 010509 | BR | Slight change to allow 18FXXK50 devices to work correctly.
 * 190711 | BR | Confirmed current version
 * 230614 | LM | Support for 18F45K50
 ********************************************************************/

#include <memory.h>
#include "pic_usb.h"
#include "pic_usb_buffer_mgt.h"


// globally accessible variables

usb_state_type usb_state = st_POWERED;

setup_data_packet usb_sdp;

uns8 usb_address;
control_mode_type control_mode;
uns16 delivery_bytes_to_send,
	  delivery_bytes_max_send,
		delivery_bytes_sent;
uns8 *delivery_ptr;
uns8 delivery_buffer_size;
uns8 *delivery_buffer;

usb_status_type usb_status;

void usb_configure_endpoints()
{

#ifdef USB_EP1
	set_bit  (uep1, EPHSHK);		// EP handshaking on
	#ifdef USB_EP1_OUT_SIZE
		set_bit(uep1, EPOUTEN);		// EP OUT enabled
	#else
		clear_bit(uep1, EPOUTEN);	// EP OUT disabled
	#endif
	#ifdef USB_EP1_IN_SIZE
		set_bit(uep1, EPINEN);		// EP IN enabled
	#else
		clear_bit(uep1, EPINEN);	// EP IN disabled
	#endif
	set_bit  (uep1, EPCONDIS);		// control transfers off

	// for IN
	#ifdef USB_EP1_IN_SIZE
		set_bit(bd1in_stat, DTS);		// turn on data togle sync TOGGLE
		clear_bit(bd1in_stat, KEN);		// clear the keep bit
		clear_bit(bd1in_stat, INCDIS);	// clear the increment disable
		clear_bit(bd1in_stat, DTSEN);
		clear_bit(bd1in_stat, BSTALL);	// clear stall bit
		clear_bit(bd1in_stat, BC9);
		clear_bit(bd1in_stat, BC8);

		clear_bit(bd1in_stat, UOWN);	// uC owns the buffer
	#endif
	// for OUT
	#ifdef USB_EP1_OUT_SIZE
		bd1out_count = USB_EP1_OUT_SIZE;
		bd1out_addr  = USB_EP1_OUT_ADDR;

		clear_bit(bd1out_stat, DTS);	// turn on data togle sync TOGGLE
		clear_bit(bd1out_stat, KEN);	// clear the keep bit
		clear_bit(bd1out_stat, INCDIS);	// clear the increment disable
		clear_bit(bd1out_stat, DTSEN);
		clear_bit(bd1out_stat, BSTALL);	// clear stall bit
		clear_bit(bd1out_stat, BC9);
		clear_bit(bd1out_stat, BC8);
		set_bit  (bd1out_stat, UOWN);	// SIE owns the buffer

	#endif
#endif

#ifdef USB_EP2
	set_bit  (uep2, EPHSHK);		// EP handshaking on
	#ifdef USB_EP2_OUT_SIZE
		set_bit(uep2, EPOUTEN);		// EP OUT enabled
	#else
		clear_bit(uep2, EPOUTEN);	// EP OUT disabled
	#endif
	#ifdef USB_EP2_IN_SIZE
		set_bit(uep2, EPINEN);		// EP IN enabled
	#else
		clear_bit(uep2, EPINEN);	// EP IN disabled
	#endif
	set_bit  (uep2, EPCONDIS);		// control transfers off

	// for IN
	#ifdef USB_EP2_IN_SIZE

		set_bit(bd2in_stat, DTS);		// turn on data togle sync TOGGLE
		clear_bit(bd2in_stat, KEN);		// clear the keep bit
		clear_bit(bd2in_stat, INCDIS);	// clear the increment disable
		clear_bit(bd2in_stat, DTSEN);
		clear_bit(bd2in_stat, BSTALL);	// clear stall bit
		clear_bit(bd2in_stat, BC9);
		clear_bit(bd2in_stat, BC8);

		clear_bit(bd2in_stat, UOWN);	// uC owns the buffer
	#endif
	// for OUT
	#ifdef USB_EP2_OUT_SIZE
		bd2out_count = USB_EP2_OUT_SIZE;
		bd2out_addr  = USB_EP2_OUT_ADDR;

		clear_bit(bd2out_stat, DTS);	// turn on data togle sync TOGGLE
		clear_bit(bd2out_stat, KEN);	// clear the keep bit
		clear_bit(bd2out_stat, INCDIS);	// clear the increment disable
		clear_bit(bd2out_stat, DTSEN);
		clear_bit(bd2out_stat, BSTALL);	// clear stall bit
		clear_bit(bd2out_stat, BC9);
		clear_bit(bd2out_stat, BC8);
		set_bit  (bd2out_stat, UOWN);	// SIE owns the buffer
	#endif
#endif

#ifdef USB_EP3
	set_bit  (uep3, EPHSHK);		// EP handshaking on
	#ifdef USB_EP3_OUT_SIZE
		set_bit(uep3, EPOUTEN);		// EP OUT enabled
	#else
		clear_bit(uep3, EPOUTEN);	// EP OUT disabled
	#endif
	#ifdef USB_EP3_IN_SIZE
		set_bit(uep3, EPINEN);		// EP IN enabled
	#else
		clear_bit(uep3, EPINEN);	// EP IN disabled
	#endif
	set_bit  (uep3, EPCONDIS);		// control transfers off
	// for IN
	#ifdef USB_EP3_IN_SIZE
		set_bit(bd3in_stat, DTS);		// ready for toggle
		clear_bit(bd3in_stat, KEN);		// clear the keep bit
		clear_bit(bd3in_stat, INCDIS);	// clear the increment disable
		clear_bit(bd3in_stat, DTSEN);
		clear_bit(bd3in_stat, BSTALL);	// clear stall bit
		clear_bit(bd3in_stat, BC9);
		clear_bit(bd3in_stat, BC8);

		clear_bit(bd3in_stat, UOWN);	// uC owns the buffer
	#endif
	// for OUT
	#ifdef USB_EP3_OUT_SIZE
		bd3out_count = USB_EP3_OUT_SIZE;
		bd3out_addr  = USB_EP3_OUT_ADDR;

		clear_bit(bd3out_stat, DTS);	// turn on data togle sync TOGGLE
		clear_bit(bd3out_stat, KEN);	// clear the keep bit
		clear_bit(bd3out_stat, INCDIS);	// clear the increment disable
		clear_bit(bd3out_stat, DTSEN);
		clear_bit(bd3out_stat, BSTALL);	// clear stall bit
		clear_bit(bd3out_stat, BC9);
		clear_bit(bd3out_stat, BC8);
		set_bit  (bd3out_stat, UOWN);	// SIE owns the buffer
	#endif

#endif

}

void usb_stall_ep0()
{
	set_bit(bd0in_stat, BSTALL);	// stall
	set_bit(bd0in_stat, UOWN);		// SIE owns the buffer
	set_bit(bd0out_stat, BSTALL);	// stall
	set_bit(bd0out_stat, UOWN);		// SIE owns the buffer
}

void usb_send_data(uns8 ep, uns8 *data, uns8 send_count, bit first)
{
#if 1 // defined(MX_USB_TYPE5) || defined (MX_USB_TYPE6)
	uns8 count;
	uns8 *bdstat;
	uns8 *bdcount;
	uns16 *bdaddr;
	uns8 *buffer;

	// this is going to be an IN transaction

	// need to grab buffer descriptor
	buffer = ep_in_buffer_location[ep];

	switch(ep) {
		case 0:
			bdstat = &bd0in_stat;
			bdcount = &bd0in_count;
			bdaddr = &bd0in_addr;
			break;
		case 1:
			bdstat = &bd1in_stat;
			bdcount = &bd1in_count;
			bdaddr = &bd1in_addr;
			break;
		case 2:
			bdstat = &bd2in_stat;
			bdcount = &bd2in_count;
			bdaddr = &bd2in_addr;
			break;
		case 3:
			bdstat = &bd3in_stat;
			bdcount = &bd3in_count;
			bdaddr = &bd3in_addr;
	}

	if (test_bit(*bdstat, UOWN))
	{
		return;
	}

	count = 0;
	while ((count < send_count))
	{
	    buffer[count] = data[count];
	    count++;
	}

	*bdcount = count;
	*bdaddr = (uns16)buffer;
	if (first)
	{
		clear_bit(*bdstat, DTS);
	}
	if (test_bit(*bdstat, DTS))
		clear_bit(*bdstat, DTS);
	else set_bit(*bdstat, DTS);
	clear_bit(*bdstat, KEN);		// clear the keep bit
	clear_bit(*bdstat, INCDIS);	// clear the increment disable
	set_bit  (*bdstat, DTSEN);
	clear_bit(*bdstat, BSTALL);	// clear stall bit
	clear_bit(*bdstat, BC9);
	clear_bit(*bdstat, BC8);

	set_bit  (*bdstat, UOWN);		// SIE owns the buffer
#else
	uns8 count;
	buffer_descriptor *bd;
	uns8 *buffer;

	// this is going to be an IN transaction

	// need to grab buffer descriptor
	buffer = ep_in_buffer_location[ep];

	bd = ep_in_bd_location[ep];

	if (test_bit(bd->stat, UOWN))
	{
		return;
	}

	count = 0;
	while ((count < send_count))
	{
	    buffer[count] = data[count];
	    count++;
	}

	bd->count = count;
	bd->addr = (uns16)buffer;
	if (first)
	{
		clear_bit(bd->stat, DTS);
	}
	if (test_bit(bd->stat, DTS))
		clear_bit(bd->stat, DTS);
	else set_bit(bd->stat, DTS);
	clear_bit(bd->stat, KEN);		// clear the keep bit
	clear_bit(bd->stat, INCDIS);	// clear the increment disable
	set_bit  (bd->stat, DTSEN);
	clear_bit(bd->stat, BSTALL);	// clear stall bit
	clear_bit(bd->stat, BC9);
	clear_bit(bd->stat, BC8);

	set_bit  (bd->stat, UOWN);		// SIE owns the buffer
#endif
}


void usb_send_data_chunk()
{

uns8 count;

	count = 0;
	while ((count < delivery_buffer_size) &&
			(delivery_bytes_sent < delivery_bytes_to_send) &&
			(delivery_bytes_sent < delivery_bytes_max_send)) {
	    delivery_buffer[count] = *delivery_ptr;
	    delivery_ptr++;
	    delivery_bytes_sent++;
	    count++;
	}

	if ((count < delivery_buffer_size) &&
			((delivery_bytes_sent == delivery_bytes_max_send) ||
			(delivery_bytes_sent == delivery_bytes_to_send))) {
		control_mode = cm_CTRL_READ_AWAITING_STATUS;	// we're done with data stage
	}

	bd0in_count = count;
	bd0in_addr = (uns16)&buffer_0_in;
	if (test_bit(bd0in_stat, DTS))
		clear_bit(bd0in_stat, DTS);
	else set_bit(bd0in_stat, DTS);
	clear_bit(bd0in_stat, KEN);		// clear the keep bit
	clear_bit(bd0in_stat, INCDIS);	// clear the increment disable
	set_bit  (bd0in_stat, DTSEN);
	clear_bit(bd0in_stat, BSTALL);	// clear stall bit
	clear_bit(bd0in_stat, BC9);
	clear_bit(bd0in_stat, BC8);

	set_bit  (bd0in_stat, UOWN);	// SIE owns the buffer
}


void usb_send_empty_data_pkt()
{
	delivery_buffer_size = USB_EP0_IN_SIZE;
	delivery_buffer = &buffer_0_in;
	delivery_bytes_sent = 0;
	delivery_bytes_to_send = 0;
	delivery_bytes_max_send = 0;
	delivery_ptr = (uns8 *) 0;
	clear_bit(bd0in_stat, DTS);		// ready to get toggled
	usb_send_data_chunk();
}

uns8 buffer_byte;

void usb_send_one_byte(uns8 data)
{
	delivery_buffer_size = USB_EP0_IN_SIZE;
	delivery_bytes_sent = 0;
	delivery_bytes_to_send = 1;
	delivery_bytes_max_send = 0;
	buffer_byte = data;
	delivery_ptr = (uns8 *) &buffer_byte;
	delivery_buffer = &buffer_0_in;

	clear_bit(bd0in_stat, DTS);		// ready to get toggled
	usb_send_data_chunk();
}

void usb_prime_ep0_out()
{
	bd0out_count = USB_EP0_OUT_SIZE;
	bd0out_addr = USB_EP0_OUT_ADDR;
	set_bit(bd0out_stat, DTS);		// turn on data togle sync TOGGLE
	clear_bit(bd0out_stat, KEN);	// clear the keep bit
	clear_bit(bd0out_stat, INCDIS);	// clear the increment disable
	set_bit  (bd0out_stat, DTSEN);
	clear_bit(bd0out_stat, BSTALL);	// clear stall bit
	clear_bit(bd0out_stat, BC9);
	clear_bit(bd0out_stat, BC8);

	set_bit  (bd0out_stat, UOWN);	// SIE owns the buffer
}

void usb_handle_standard_request(setup_data_packet sdp)
{
	switch (sdp.bRequest)
	{
		case req_Get_Descriptor:
			uns8 descriptor_type = sdp.wValue >> 8; 	// high byte is descriptor
			uns8 descriptor_num  = sdp.wValue & 0xff; 	// low byte is particular descriptor

			usb_get_descriptor_callback(descriptor_type, descriptor_num, &delivery_ptr, &delivery_bytes_to_send);
			if (delivery_ptr != 0)
			{	// we've got something
				control_mode = cm_CTRL_READ_DATA_STAGE;
				delivery_bytes_max_send = sdp.wLength;	// maximum host wants
				delivery_bytes_sent = 0;				// clear our sent countr
				delivery_buffer_size = USB_EP0_IN_SIZE;
				delivery_buffer = (uns8 *)USB_EP0_IN_ADDR;
				clear_bit(bd0in_stat, DTS);				// ready to get toggled
				usb_send_data_chunk();
			}
			else
			{
				usb_stall_ep0();
			}

			break;
		case req_Set_Address:
			usb_address = sdp.wValue & 0xff;
			usb_status = us_SET_ADDRESS;

			// Send a status ack - when we confirm that, THEN change address
			usb_send_status_ack();
			control_mode = cm_CTRL_WRITE_SENDING_STATUS;
			break;
		case req_Set_Configuration:
			control_mode = cm_CTRL_WRITE_SENDING_STATUS;
			usb_send_status_ack();
			usb_state = st_CONFIGURED;
			// device is up - so now get the endpoints happy
			usb_configure_endpoints();
			// and do a call back to let the app know we're ready
			#ifdef USB_CALLBACK_ON_DEVICE_CONFIGURED
				usb_device_configured_callback();
			#endif
			break;
		case req_Get_Interface:
				control_mode = cm_CTRL_READ_DATA_STAGE;
				usb_send_one_byte(1);
				break;
		case req_Get_Status:
				#ifdef USB_SELF_POWERED
					usb_send_one_byte(1);
				#else
					usb_send_one_byte(0);	// bus powered
				#endif
				break;
		default:
			usb_stall_ep0();
			break;

	}
}


void usb_handle_transaction()
{
	uns8 end_point, pid;
	end_point = ustat >> 3;

	//read USTAT for endpoint information
	if (test_bit(ustat, DIR))
	{
		pid = (bd0in_stat >> 2) & 0x0f;		// mask out pid
	}
	else
	{
		pid = (bd0out_stat >> 2) & 0x0f;	// mask out pid
	}

	if (end_point == 0)
	{
		if (pid == pid_SETUP)
		{
			memcpy(/*dst*/ (void*)&usb_sdp, /*src*/ (void *)&buffer_0_out, 8);
			usb_prime_ep0_out();

			if (!test_bit(usb_sdp.bmRequestType, REQUEST_TYPE1) && // std request
				!test_bit(usb_sdp.bmRequestType, REQUEST_TYPE0)) {
				usb_handle_standard_request(usb_sdp);
			}
			else if (!test_bit(usb_sdp.bmRequestType, REQUEST_TYPE1) && // class request
				test_bit(usb_sdp.bmRequestType, REQUEST_TYPE0)) {
				#ifdef USB_CALLBACK_ON_CTRL_CLASS
					usb_handle_class_request(usb_sdp);
				#endif
			}
			clear_bit(ucon, PKTDIS);
		}
		else if (pid == pid_IN)
		{
			if (control_mode == cm_CTRL_READ_DATA_STAGE)
			{
				// it's ours, so send next chunk
				usb_send_data_chunk();
			}
			else if (control_mode == cm_CTRL_WRITE_SENDING_STATUS)
			{
				control_mode = cm_IDLE;
				if (usb_status == us_SET_ADDRESS)
				{
					usb_state = st_ADDRESS;
					uaddr = usb_address;
					usb_status = us_IDLE;
				}
			}
			else if (control_mode == cm_CTRL_READ_AWAITING_STATUS)
			{
				// Must have been last IN of the read, so still waiting for status
				nop(); // boostc bug
			}
			else if (control_mode == cm_CTRL_READ_DATA_STAGE_CLASS)
			{
				// Must be more to come
				#ifdef USB_CALLBACK_ON_CTRL_CLASS
					usb_handle_ctrl_read_class();
				#else
					nop(); // otherwise boostc bug
				#endif
			}
		}
		else if (pid == pid_ACK)
		{
			if (control_mode == cm_CTRL_READ_DATA_STAGE)
			{
				usb_send_data_chunk();
			}
			else if (control_mode == cm_CTRL_READ_DATA_STAGE_CLASS)
			{
				#ifdef USB_CALLBACK_ON_CTRL_CLASS
					usb_handle_ctrl_read_class();
				#else
					nop(); // boostc bug
				#endif
			}
			else if (control_mode == cm_CTRL_WRITE_SENDING_STATUS)
			{
				control_mode = cm_IDLE;
			}

		}
		else if (pid == pid_OUT)
		{
			// We've done an out
			if (control_mode == cm_CTRL_READ_AWAITING_STATUS)
			{
				control_mode = cm_IDLE;
			}
			else if (control_mode == cm_CTRL_WRITE_DATA_STAGE_CLASS)
			{
				#ifdef USB_CALLBACK_ON_CTRL_CLASS
					usb_handle_ctrl_write_class((uns8 *)&buffer_0_out, bd0out_count);
					// !! should include bc bits here for total count
				#else
					nop();
				#endif
			}
			else if (control_mode == cm_CTRL_READ_DATA_STAGE)
			{
				control_mode = cm_IDLE;
			}
			usb_prime_ep0_out();
		}

	}
	else
	{
		buffer_descriptor *bd;
		if (test_bit(ustat, DIR))
		{
			// in
			#ifdef USB_EP_DATA_CALLBACK
#if 1 //defined(MX_USB_TYPE5) || defined (MX_USB_TYPE6)
				{
					uns8 count;
					switch (end_point) {
						case 0: count = bd0in_count;
							break;
						case 1: count = bd1in_count;
							break;
						case 2: count = bd2in_count;
							break;
						case 3: count = bd3in_count;
					}
					usb_ep_data_in_callback(end_point, count);
				}
#else
				bd = ep_in_bd_location[end_point];
				usb_ep_data_in_callback(end_point, bd->count);
#endif
			#else
				nop();
			#endif
		}
		else
		{
			// out
#if 1 //defined(MX_USB_TYPE5) || defined (MX_USB_TYPE6)
			uns8 *bdstat;
			uns8 *bdcount;

			switch (end_point) {
				case 0:
					bdstat = &bd0out_stat;
					bdcount = &bd0out_count;
					break;
				case 1:
					bdstat = &bd1out_stat;
					bdcount = &bd1out_count;
					break;
				case 2:
					bdstat = &bd2out_stat;
					bdcount = &bd2out_count;
					break;
				case 3:
					bdstat = &bd3out_stat;
					bdcount = &bd3out_count;
			}
			#ifdef USB_EP_DATA_CALLBACK
				usb_ep_data_out_callback(end_point, ep_out_buffer_location[end_point],
									*bdcount);
			#endif

			// re-prime endpoint
			*bdcount = ep_out_buffer_size[end_point];

			// Address shouldn't change, so don't need to update it

			clear_bit(*bdstat, DTS);		// turn on data togle sync TOGGLE
			clear_bit(*bdstat, KEN);		// clear the keep bit
			clear_bit(*bdstat, INCDIS);	// clear the increment disable
			clear_bit(*bdstat, DTSEN);
			clear_bit(*bdstat, BSTALL);	// clear stall bit
			clear_bit(*bdstat, BC9);
			clear_bit(*bdstat, BC8);
			set_bit  (*bdstat, UOWN);		// SIE owns the buffer
#else
			bd = ep_out_bd_location[end_point];
			// issue callback
			#ifdef USB_EP_DATA_CALLBACK
				usb_ep_data_out_callback(end_point, ep_out_buffer_location[end_point],
									bd->count);
			#endif
			// re-prime endpoint
			bd->count = ep_out_buffer_size[end_point];

			// Address shouldn't change, so don't need to update it

			clear_bit(bd->stat, DTS);		// turn on data togle sync TOGGLE
			clear_bit(bd->stat, KEN);		// clear the keep bit
			clear_bit(bd->stat, INCDIS);	// clear the increment disable
			clear_bit(bd->stat, DTSEN);
			clear_bit(bd->stat, BSTALL);	// clear stall bit
			clear_bit(bd->stat, BC9);
			clear_bit(bd->stat, BC8);
			set_bit  (bd->stat, UOWN);		// SIE owns the buffer
#endif
		}
	}
}


void usb_handle_reset()
{
	usb_address = 0;
	//uaddr = 0;

	control_mode = cm_IDLE;
	usb_status   = us_IDLE;

	// clear fifo
	clear_bit(uir, TRNIF);
	clear_bit(uir, TRNIF);
	clear_bit(uir, TRNIF);
	clear_bit(uir, TRNIF);

	// init buffers

	// EP0 OUT
	bd0out_count = USB_EP0_OUT_SIZE;	// 8 byte buffer
	bd0out_addr = USB_EP0_OUT_ADDR;

	clear_bit(bd0out_stat, DTS);	// turn on data togle sync TOGGLE
	clear_bit(bd0out_stat, KEN);	// clear the keep bit
	clear_bit(bd0out_stat, INCDIS);	// clear the increment disable
	clear_bit  (bd0out_stat, DTSEN);	// !!!!!!
	clear_bit(bd0out_stat, BSTALL);	// clear stall bit
	clear_bit(bd0out_stat, BC9);
	clear_bit(bd0out_stat, BC8);

	set_bit  (bd0out_stat, UOWN);	// SIE owns the buffer
					// since we expect frist transaction to be SETUP
	// EP0 IN
	bd0in_count = USB_EP0_IN_SIZE;	// 8 byte buffer
	bd0in_addr = USB_EP0_IN_ADDR;
	clear_bit(bd0in_stat, DTS);	// turn on data togle sync TOGGLE
	clear_bit(bd0in_stat, KEN);	// clear the keep bit
	clear_bit(bd0in_stat, INCDIS);	// clear the increment disable
	clear_bit(bd0in_stat, BSTALL);	// clear stall bit
	clear_bit(bd0in_stat, BC9);
	clear_bit(bd0in_stat, BC8);

	clear_bit(bd0in_stat, UOWN);	// uC owns the buffer
}


void usb_handle_stall()
{
}


void usb_handle_isr()
{
	if (test_bit(PIRREG, USBIF))			//USB Interrupt?
	{
		clear_bit(PIRREG, USBIF);			//Clear USB Interrupt

		if (test_bit(uir, IDLEIF))		//USB Idle Interrupt?
		{
			clear_bit(uir, IDLEIF);		//Clear Idle
		}

		if (test_bit(uir, TRNIF))		//USB Transaction detected?
		{
			usb_handle_transaction();	//Handle the transaction
			clear_bit(uir, TRNIF);		//Clear the interrupt
		}

		if (test_bit(uir, URSTIF))		//USB Reset detected?
		{
			usb_handle_reset();			//Deal with the reset
			clear_bit(uir, URSTIF);		//Clear the interrupt
		}

		if (test_bit(uir, STALLIF))		//USB Stall detected?
		{
			usb_handle_stall();			//Handle the stall
			clear_bit(uir, STALLIF);	//Clear the interrupt
		}
		if (test_bit(uir, SOFIF))		//USB Start of frame detected?
		{
			#ifdef USB_CALLBACK_ON_SOF
				usb_SOF_callback(ufrml);	// sourceboost defines this
			#endif
 			clear_bit(uir, SOFIF);		//Clear the interrupt
		}
	}
}


void usb_setup()
{
	usb_state = st_POWERED;

	// init hardware
        #if (USB_DESCRIPTOR_ADDRESS != 0x2000)
	// JK: Bit is reserved for 16f1455 series, do not touch
        clear_bit(ucfg, 3);     // enable internal tranceiver   -- BR Modded from UTRDIS to 3 for K50 devices
	#endif
	set_bit(ucfg, FSEN);	// clear for low speed, set for high speed
	set_bit  (ucfg, UPUEN);	// enable on-chip pull-ups

	clear_bit(ucfg, PPB1);	// disable double buffering for now
	clear_bit(ucfg, PPB0);

	// init endpoints
	set_bit(uep0,   EPHSHK);	// EP0 handshaking on
	set_bit(uep0,   EPOUTEN);	// EP0 OUT enable
	set_bit(uep0,   EPINEN); 	// EP0 IN enable
	clear_bit(uep0, EPCONDIS);	// EP0 control transfers on (and IN and OUT)

	// init interrupts
	set_bit(uie,  STALLIE);		// interrupt on stall
	set_bit(uie,  TRNIE);		// on transaction complete
	set_bit(uie,  URSTIE);		// on reset
	set_bit(PIEREG, USBIE);		// general USB interrupts
	#ifdef USB_CALLBACK_ON_SOF
		set_bit(uie, SOFIE);
	#endif

}

void usb_enable_module()
{
	// enable usb module
	uir = 0;
	set_bit(ucon, USBEN);		// enable USB serial interface engine (SIE)
	usb_state = st_DEFAULT;
}


usb_state_type usb_get_state()
{
	return usb_state;
}
