/*********************************************************************
 *                    Flowcode USB Component Code
 *
 * File: pic_usb_buffer_mgt.c
 *
 * (c) 2009 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 031008 | BR | Created
 * 190711 | BR | Confirmed current version
 * 250613 | JK | Added support for 16f1455
 * 230614 | LM | Support for 18F45K50
 ********************************************************************/

#include "pic_usb_buffer_mgt.h"

#if (USB_DESCRIPTOR_BASE_ADDRESS == 0x200)
	#define USBEP0O		0x200
	#define USBEP0I		0x204
	#define USBEP1O		0x208
	#define USBEP1I		0x20C
	#define USBEP2O		0x210
	#define USBEP2I		0x214
	#define USBEP3O		0x218
	#define USBEP3I		0x21C
#endif

#if (USB_DESCRIPTOR_BASE_ADDRESS == 0x400)
	#define USBEP0O		0x400
	#define USBEP0I		0x404
	#define USBEP1O		0x408
	#define USBEP1I		0x40C
	#define USBEP2O		0x410
	#define USBEP2I		0x414
	#define USBEP3O		0x418
	#define USBEP3I		0x41C
#endif

#if (USB_DESCRIPTOR_BASE_ADDRESS == 0xd00)
	#define USBEP0O		0xd00
	#define USBEP0I		0xd04
	#define USBEP1O		0xd08
	#define USBEP1I		0xd0C
	#define USBEP2O		0xd10
	#define USBEP2I		0xd14
	#define USBEP3O		0xd18
	#define USBEP3I		0xd1C
#endif

#if (USB_DESCRIPTOR_BASE_ADDRESS == 0x2000)
	#define USBEP0O		0x2000
	#define USBEP0I		0x2004
	#define USBEP1O		0x2008
	#define USBEP1I		0x200C
	#define USBEP2O		0x2010
	#define USBEP2I		0x2014
	#define USBEP3O		0x2018
	#define USBEP3I		0x201C
#endif

#if (USB_DESCRIPTOR_BASE_ADDRESS == 0x2000)
// JK: Work around BoostC issue with FSR access only for arrays
uns8	bd0aout_stat[1]@0x2000;
uns8	bd0aout_count[1]@0x2001;
uns16	bd0aout_addr[1]@0x2002;
uns8	bd0ain_stat[1]@0x2004;
uns8	bd0ain_count[1]@0x2005;
uns16	bd0ain_addr[1]@0x2006;

uns8	bd1aout_stat[1]@0x2008;
uns8	bd1aout_count[1]@0x2009;
uns16	bd1aout_addr[1]@0x200a;
uns8	bd1ain_stat[1]@0x200c;
uns8	bd1ain_count[1]@0x200d;
uns16	bd1ain_addr[1]@0x200e;

uns8	bd2aout_stat[1]@0x2010;
uns8	bd2aout_count[1]@0x2011;
uns16	bd2aout_addr[1]@0x2012;
uns8	bd2ain_stat[1]@0x2014;
uns8	bd2ain_count[1]@0x2015;
uns16	bd2ain_addr[1]@0x2016;

uns8	bd3aout_stat[1]@0x2018;
uns8	bd3aout_count[1]@0x2019;
uns16	bd3aout_addr[1]@0x201a;
uns8	bd3ain_stat[1]@0x201c;
uns8	bd3ain_count[1]@0x201d;
uns16	bd3ain_addr[1]@0x201e;

#else
buffer_descriptor bd0out@USBEP0O;
buffer_descriptor bd0in @USBEP0I;
buffer_descriptor bd1out@USBEP1O;
buffer_descriptor bd1in @USBEP1I;
buffer_descriptor bd2out@USBEP2O;
buffer_descriptor bd2in @USBEP2I;
buffer_descriptor bd3out@USBEP3O;
buffer_descriptor bd3in @USBEP3I;
#endif


uns8 buffer_0_out[USB_EP0_OUT_SIZE]@ USB_EP0_OUT_ADDR;
uns8 buffer_0_in [USB_EP0_IN_SIZE] @ USB_EP0_IN_ADDR;

#ifdef USB_EP1_IN_SIZE
	uns8 buffer_1_in [USB_EP1_IN_SIZE] @USB_EP1_IN_ADDR;
#endif
#ifdef USB_EP1_OUT_SIZE
	uns8 buffer_1_out [USB_EP1_OUT_SIZE] @ USB_EP1_OUT_ADDR;
#endif
#ifdef USB_EP2_IN_SIZE
	uns8 buffer_2_in [USB_EP2_IN_SIZE] @ USB_EP2_IN_ADDR;
#endif
#ifdef USB_EP2_OUT_SIZE
	uns8 buffer_2_out [USB_EP2_OUT_SIZE] @ USB_EP2_OUT_ADDR;
#endif
#ifdef USB_EP3_IN_SIZE
	uns8 buffer_3_in [USB_EP3_IN_SIZE] @ USB_EP3_IN_ADDR;
#endif
#ifdef USB_EP3_OUT_SIZE
	uns8 buffer_3_out [USB_EP3_OUT_SIZE] @ USB_EP3_OUT_ADDR;
#endif

#if 0 // !defined(MX_USB_TYPE5) && !defined (MX_USB_TYPE6)
buffer_descriptor * ep_in_bd_location[USB_HIGHEST_EP + 1];
buffer_descriptor * ep_out_bd_location[USB_HIGHEST_EP + 1];
#endif

uns8 * ep_in_buffer_location[USB_HIGHEST_EP + 1] = {
	// we always have EP0

	&buffer_0_in,	// ep 0 in buffer location

	#if USB_HIGHEST_EP >= 1
		#ifdef USB_EP1_IN_SIZE
			&buffer_1_in,	// ep 1 in buffer location
		#else
			0,
		#endif
	#endif

	#if USB_HIGHEST_EP >= 2
		#ifdef USB_EP2_IN_SIZE
			&buffer_2_in,	// ep 2 in buffer location
		#else
			0,
		#endif
	#endif

	#if USB_HIGHEST_EP >= 3
		#ifdef USB_EP3_IN_SIZE
			&buffer_3_in,	// ep 3 in buffer location
		#else
			0,
		#endif
	#endif
};

uns8 * ep_out_buffer_location[USB_HIGHEST_EP + 1] = {
	// we always have EP0

	&buffer_0_out,	// ep 0 in buffer location

	#if USB_HIGHEST_EP >= 1
		#ifdef USB_EP1_OUT_SIZE
			&buffer_1_out,	// ep 1 in buffer location
		#else
			0,
		#endif
	#endif

	#if USB_HIGHEST_EP >= 2
		#ifdef USB_EP2_OUT_SIZE
			&buffer_2_out,	// ep 2 in buffer location
		#else
			0,
		#endif
	#endif

	#if USB_HIGHEST_EP >= 3
		#ifdef USB_EP3_OUT_SIZE
			&buffer_3_out,	// ep 3 in buffer location
		#else
			0,
		#endif
	#endif
};


uns16 ep_in_buffer_size[USB_HIGHEST_EP + 1] = {
	USB_EP0_IN_SIZE,

	#if USB_HIGHEST_EP >= 1
		#ifdef USB_EP1_IN_SIZE
			USB_EP1_IN_SIZE,
		#else
			0,
		#endif
	#endif

	#if USB_HIGHEST_EP >= 2
		#ifdef USB_EP2_IN_SIZE
			USB_EP2_IN_SIZE,
		#else
			0,
		#endif
	#endif

	#if USB_HIGHEST_EP >= 3
		#ifdef USB_EP3_IN_SIZE
			USB_EP3_IN_SIZE,
		#else
			0,
		#endif
	#endif
};



uns16 ep_out_buffer_size[USB_HIGHEST_EP + 1] = {
	USB_EP0_OUT_SIZE,

	#if USB_HIGHEST_EP >= 1
		#ifdef USB_EP1_OUT_SIZE
			USB_EP1_OUT_SIZE,
		#else
			0,
		#endif
	#endif

	#if USB_HIGHEST_EP >= 2
		#ifdef USB_EP2_OUT_SIZE
			USB_EP2_OUT_SIZE,
		#else
			0,
		#endif
	#endif

	#if USB_HIGHEST_EP >= 3
		#ifdef USB_EP3_OUT_SIZE
			USB_EP3_OUT_SIZE,
		#else
			0,
		#endif
	#endif
};
