/*********************************************************************
 *                    Flowcode USB Component Code
 *
 * File: pic_utils.h
 *
 * (c) 2009 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 031008 | BR | Created
 * 190711 | BR | Confirmed current version
 ********************************************************************/

#ifndef __pic_utils_h
#define __pic_utils_h

#include <system.h>


// Commonly used data types, to make everything more portable

#define int8  char
#define uns8  unsigned char
#define uns16 unsigned int
#define int16 int
#define uns32 unsigned long
#define int32 long


#define set_pin(port, pin) \
 set_bit(port_shadow[port - PORTA], pin); \
 port_array[port - PORTA] = port_shadow[port - PORTA];

#define clear_pin(port, pin) \
 clear_bit(port_shadow[port - PORTA], pin); \
 port_array[port - PORTA] = port_shadow[port - PORTA];

void set_pin_var(uns8 port, uns8 pin);
void clear_pin_var(uns8 port, uns8 pin);

#define turn_peripheral_ints_on() set_bit(intcon, PEIE)
#define turn_peripheral_ints_off() clear_bit(intcon, PEIE)

#define turn_global_ints_on() set_bit(intcon, GIE)
#define turn_global_ints_off() clear_bit(intcon, GIE)

// If you want to make sure the interrupts are dead, this is the best way

#define kill_interrupts()    do { \
   	 bit store_gie; \
   	 intcon.GIE = 0; \
   } while (intcon.GIE != 0)

// Start of critical section - turn off all interrupts
#define start_crit_sec() \
	bit store_gie = intcon.GIE; \
    kill_interrupts()

// End of critical section, restore interrupts if they were on before
#define end_crit_sec() \
	intcon.GIE = store_gie

#endif