/*********************************************************************
 *                    Flowcode USB Component Code
 *
 * File: usb_cdc_class.c
 *
 * (c) 2009 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 031008 | BR | Created
 * 190711 | BR | Confirmed current version
 * 230614 | LM | Support for 18F45K50
 ********************************************************************/

// USB includes
#include "pic_usb.h"
#include "pic_usb_buffer_mgt.h"

// local includes
#include "usb_cdc_class.h"

// system includes
#include <memory.h>

// class definitions

#define req_SEND_ENCAPSULATED_COMMAND	0x00
#define req_GET_ENCAPSULATED_RESPONSE	0x01
#define	req_SET_COMM_FEATURE			0x02
#define req_GET_COMM_FEATURE			0x03
#define req_CLEAR_COMM_FEATURE			0x04
#define req_SET_LINE_CODING				0x20
#define req_GET_LINE_CODING				0x21
#define req_SET_CONTROL_LINE_STATE		0x22
#define req_SEND_BREAK					0x23

typedef  union _long_union{
		long as_long;
		uns8 as_byte_array[4];
} long_union;


typedef struct _line_coding {
	long_union	dte_rate;
	uns8	stop_bits;	// 0=1 stop bit, 1=1.5 stop bits, 2=2 stop bits
	uns8	parity;		// 0=None, 1=Odd, 2=Even, 3=Mark, 4=Space
	uns8	data_bits;	// 5,6,7,8 or 16 bits
}	line_coding;

/** Transmit fifo  */
uns8 cdc_tx_buffer[CDC_TX_BUFFER_SIZE];
/** Transmit fifo start point */
uns8 cdc_tx_start=0;
/** Transmit fifo end point */
uns8 cdc_tx_end=0;

/** Receive fifo */
uns8 cdc_rx_buffer[CDC_RX_BUFFER_SIZE];
/** Receive fifo start point */
uns8 cdc_rx_start = 0;
/** Receive fifo end point */
uns8 cdc_rx_end = 0;


uns8 class_data[8];	// we'll dump all our class data in here

void usb_handle_class_request(setup_data_packet sdp) {

	switch (sdp.bRequest) {
		case req_SET_LINE_CODING:
			// we now expect the line coding to arrive in the data stage
			control_mode = cm_CTRL_WRITE_DATA_STAGE_CLASS;
			break;
		case req_GET_LINE_CODING:
			control_mode = cm_CTRL_READ_DATA_STAGE_CLASS;
			//  need to prime ep0 IN with some funky data here
			usb_send_data(/*ep*/ 0, /*data*/ &class_data, /*count*/ 8, /*first*/ 1);
			// actually we know this will be the last packet, so go straight to waiting for the status ack
			break;
		case req_SET_CONTROL_LINE_STATE:
			// no data, so just ack the status
			control_mode = cm_CTRL_WRITE_SENDING_STATUS;
			usb_send_status_ack();
			break;
		default:
	}
}

void usb_handle_ctrl_write_class(uns8 *data, uns16 count) {

	switch (usb_sdp.bRequest) {
		case req_SET_LINE_CODING:
			// dump it into class_data
			memcpy(/* dst */ (void *)&class_data,/* src */ (void *)data, count);

			// Now we need to send an ACK status back
			control_mode = cm_CTRL_WRITE_SENDING_STATUS;
			usb_send_status_ack();
			line_coding *my_lc;
			my_lc = (line_coding*) &class_data;
			break;
	}
}

void usb_handle_ctrl_read_class()
{
		switch (usb_sdp.bRequest)
		{
			case req_GET_LINE_CODING:
				// we know we've already sent everything, so now wait for status
				control_mode = cm_CTRL_READ_AWAITING_STATUS;
				break;
			default:
		}

}

void usb_ep_data_out_callback(uns8 end_point, uns8 *buffer, uns16 byte_count)
{
	uns8 cdc_rx_next;

	// We have some data!
	if (end_point == CDC_DATA_ENDPOINT)
	{	// it's the data end point

		uns8 count;
		for (count = 0; count < byte_count; count++)
		{
			cdc_rx_next = cdc_rx_end + 1;	// get next buffer position
			if (cdc_rx_next == CDC_RX_BUFFER_SIZE)
			{	// if we're at the end
				cdc_rx_next = 0;	// then wrap to the beginning
			}
			if (cdc_rx_next != cdc_rx_start)
			{ // if space in the fifo
				cdc_rx_buffer[cdc_rx_end] = buffer[count]; // put it in
				cdc_rx_end = cdc_rx_next;  // and move pointer along
			} // else... just ignore it, we've lost a byte, no room in the inn
		}
	}
}

void usb_ep_data_in_callback(uns8 end_point, uns16 byte_count) {
	// data has been sent, so do we need to send more?
	usb_cdc_handle_tx();
}


void usb_ep_get_rx_buffer(char length, char *buffer)
{
	uns8 count;

	if (length > 64)
		length = 64;		// check for buffer overflow length

	for (count = 0; count < length; count++)
	{
		buffer[count] = cdc_rx_buffer[count];
	}
	cdc_rx_end = cdc_rx_start;
}


void usb_cdc_putc(uns8 c) {
uns8 cdc_tx_next;
bit  my_store_gie;
#ifdef CDC_IDE_DEBUG
return;
#endif

	cdc_tx_next = cdc_tx_end + 1;	// get next buffer position
	if (cdc_tx_next == CDC_TX_BUFFER_SIZE) {	// if we're at the end
		cdc_tx_next = 0;	// wrap to the beginning
	}

	if ((!intcon.GIE) && (cdc_tx_next == cdc_tx_start)) {
		return;
	}
	while (cdc_tx_next == cdc_tx_start) {
	}
	my_store_gie = intcon.GIE;	// store interrupt state
	kill_interrupts();	// turn off global interrupts

	cdc_tx_buffer[cdc_tx_end] = c; // put it in
	cdc_tx_end = cdc_tx_next;  // move pointer along

	intcon.GIE = my_store_gie;	// restore interrupt state

}


void usb_cdc_handle_tx()
{
uns8 cdc_tx_next;
uns8 count;
uns16 buffer_size;
uns8 *buffer;
buffer_descriptor *bd;

#if 1  // defined(MX_USB_TYPE5) || defined (MX_USB_TYPE6)
	#if   (CDC_DATA_ENDPOINT == 1)
		bd = &bd1in;
	#elif (CDC_DATA_ENDPOINT == 2)
		bd = &bd2in;
	#elif (CDC_DATA_ENDPOINT == 3)
		bd = &bd3in;
	#else
		bd = &bd0in;
	#endif
#else
	bd = ep_in_bd_location[CDC_DATA_ENDPOINT];
#endif

	if (test_bit(bd->stat, UOWN)) {	// if there's already something in play
		return;	// give up
	}

	buffer_size = ep_in_buffer_size[CDC_DATA_ENDPOINT];
	buffer = ep_in_buffer_location[CDC_DATA_ENDPOINT];

	if (cdc_tx_end == cdc_tx_start) { // anything in the fifo?
	   return; // nope
	}

	start_crit_sec();

	count = 0;
	while ((cdc_tx_end != cdc_tx_start) && (count < buffer_size)) {

		cdc_tx_next = cdc_tx_start + 1;	// get next position
		if (cdc_tx_next == CDC_TX_BUFFER_SIZE) {	// if we're at the end of the buffer
			cdc_tx_next = 0;	// wrap to the beginning
		}
		buffer[count] = cdc_tx_buffer[cdc_tx_start];	// transmit the character
		count++;
		cdc_tx_start = cdc_tx_next;	// move start position of fifo
	}
	if (count > 0) {
		bd->count = count;
		bd->addr = (uns16)buffer;
		if(test_bit(bd->stat, DTS))
			clear_bit(bd->stat, DTS);
		else set_bit(bd->stat, DTS);
		clear_bit(bd->stat, KEN);	// clear the keep bit
		clear_bit(bd->stat, INCDIS);	// clear the increment disable
		set_bit  (bd->stat, DTSEN);
		clear_bit(bd->stat, BSTALL);	// clear stall bit
		clear_bit(bd->stat, BC9);
		clear_bit(bd->stat, BC8);

		set_bit  (bd->stat, UOWN);	// SIE owns the buffer
	}
	end_crit_sec();
}

uns8 usb_cdc_rx_avail() { return (cdc_rx_end - cdc_rx_start); }
uns8 usb_cdc_tx_empty() { return cdc_tx_start == cdc_tx_end; }

void usb_cdc_print_str(char *str) {

uns8 count;
buffer_descriptor *bd;

	for(count = 0 ; str[count] != 0; count++)
    {
        usb_cdc_putc(str[count]);
    }
}

void usb_SOF_callback(uns16 frame) {
	// we don't care about the frame number, we only care if there's something to send...
	usb_cdc_handle_tx();	// start transmission
}

void usb_cdc_setup() {
	line_coding *my_lc;
	my_lc = (line_coding*) &class_data;
	my_lc->dte_rate.as_long = 0x60090000;
	my_lc->stop_bits = 0;
	my_lc->data_bits = 8;
	my_lc->parity = 0;
}
