/*********************************************************************
 *                    Flowcode USB Component Code
 *
 * File: usb_dll_class.c
 *
 * (c) 2009 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 031008 | BR | Created
 * 190711 | BR | Confirmed current version
 ********************************************************************/



// USB includes
#include "pic_usb.h"
#include "pic_usb_buffer_mgt.h"


// local includes
#include "usb_dll_class.h"

// system includes
#include "memory.h"

#define req_GET_REPORT		0x01
#define req_GET_IDLE		0x02
#define req_GET_PROTOCOL	0x03
#define req_SET_REPORT		0x09
#define req_SET_IDLE		0x0a
#define req_SET_PROTOCOL	0x0b


// Variable to hold outgoing data PC point of view
extern char DLL_USB_Rx_String[MX_OUT_PCK_SIZE];

/** Receive fifo start point */
uns8 dll_rx_start = 0;
/** Receive fifo end point */
uns8 dll_rx_end = 0;


void usb_handle_ctrl_read_class()
{
}

void usb_handle_ctrl_write_class(uns8 *data, uns16 count)
{
}

void usb_handle_class_request(setup_data_packet sdp)
{
	switch(sdp.bRequest)
	{
		case req_GET_REPORT:
			break;
		case req_GET_IDLE:
			break;
		case req_GET_PROTOCOL:
			break;
		case req_SET_REPORT:
			break;
		case req_SET_IDLE:
			// we don't support whatever they want
			usb_stall_ep0();
			break;
		case req_SET_PROTOCOL:
			break;
	}
}


void usb_ep_data_out_callback(uns8 end_point, uns8 *buffer, uns16 byte_count)
{
	uns8 dll_rx_next;
							// We have some data!
	if (end_point == 1)		// is it the data end point?
	{
		uns8 count;
		for (count = 0; count < byte_count; count++)
		{
			dll_rx_next = dll_rx_end + 1;			// get next buffer position
			if (dll_rx_next == MX_OUT_PCK_SIZE)		// if we're at the end
			{
				dll_rx_next = 0;					// then wrap to the beginning
			}
			if (dll_rx_next != dll_rx_start)		// if space in the fifo
			{
				DLL_USB_Rx_String[dll_rx_end] = buffer[count]; 	// put it in
				dll_rx_end = dll_rx_next;  						// and move pointer along
			}
		}
	}
}



void usb_ep_data_in_callback(uns8 end_point, uns16 byte_count)
{
}



char CheckRx(void)
{
	char length;

	if(dll_rx_start != dll_rx_end)
	{
		length = dll_rx_end - dll_rx_start;
		dll_rx_end = 0;
		return length;
	}
	return 0;
}


void SendString(char* String, char Length)
{
	usb_send_data(1, String, Length, 0);			//Send string to host
}

