/*********************************************************************
 *                    Flowcode USB Component Code
 *
 * File: usb_hid_class.c
 *
 * (c) 2009 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 031008 | BR | Created
 * 190711 | BR | Confirmed current version
 * 190113 | JK | Added extended USB capabilities & Callback get report
 ********************************************************************/


// USB includes
#include "pic_usb.h"
#include "pic_usb_buffer_mgt.h"


// local includes
#include "usb_hid_class.h"

// system includes
#include "memory.h"

#define req_GET_REPORT		0x01
#define req_GET_IDLE		0x02
#define req_GET_PROTOCOL	0x03
#define req_SET_REPORT		0x09
#define req_SET_IDLE		0x0a
#define req_SET_PROTOCOL	0x0b


// Variable to hold status of incoming data PC point of view
extern char outgoing;

// Variable to hold outgoing data PC point of view
#if MX_OUT_PCK_SIZE > 0
	extern char HID_USB_Rx_String[MX_OUT_PCK_SIZE];
	extern char incoming;
#endif


/** Receive fifo start point */
uns8 hid_rx_start = 0;
/** Receive fifo end point */
uns8 hid_rx_end = 0;



void usb_handle_ctrl_read_class()
{
}

void usb_handle_ctrl_write_class(uns8 *data, uns16 count)
{
}

void usb_handle_class_request(setup_data_packet sdp)
{
	switch(sdp.bRequest)
	{
		case req_GET_REPORT:
#ifdef  FCV_EXTENDED_USB
#ifdef	FCV_HID_REPORT_CALLBACK
			uns8 report_type = sdp.wValue >> 8;         // high byte is report type
                        uns8 report_num  = sdp.wValue & 0xff;       // low byte is particular report
			uns8 result;

			result = FCM_GetReport(report_type, report_num);
                        if (result != 0)
                        {       // we've got something
				delivery_ptr = &FCV_USBREPORT[0];
				delivery_bytes_to_send = FCV_USBREPORT_SIZE;
                                control_mode = cm_CTRL_READ_DATA_STAGE;
                                delivery_bytes_max_send = sdp.wLength;  // maximum host wants
                                delivery_bytes_sent = 0;                                // clear our sent countr
                                delivery_buffer_size = USB_EP0_IN_SIZE;
                                delivery_bd = &bd0in;
                                delivery_buffer = (uns8 *)USB_EP0_IN_ADDR;
                                clear_bit(bd0in.stat, DTS);                             // ready to get toggled
                                usb_send_data_chunk();
                        }
                        else
                        {
                                usb_stall_ep0();
                        }
#endif
#endif			
			break;
		case req_GET_IDLE:
			break;
		case req_GET_PROTOCOL:
			break;
		case req_SET_REPORT:
			break;
		case req_SET_IDLE:
			// we don't support whatever they want
			usb_stall_ep0();
			break;
		case req_SET_PROTOCOL:
			break;
	}
}


void usb_ep_data_out_callback(uns8 end_point, uns8 *buffer, uns16 byte_count)
{
	char i;

	#if MX_OUT_PCK_SIZE > 0										//If we are expecting data in

	incoming = byte_count;

		for (i=0; i<byte_count; i++)							//Loop through each data byte
		{
			HID_USB_Rx_String[i] = buffer[i];					//Storing incoming data to local buffer
		}

		if (i < MX_OUT_PCK_SIZE)								//If we did not receive all we expected
		{
			for (i=byte_count; i<MX_OUT_PCK_SIZE; i++)
			{
				HID_USB_Rx_String[i] = 0;						//Clear the rest of the buffer
			}
		}
	#endif
}


void usb_ep_data_in_callback(uns8 end_point, uns16 byte_count)
{
	outgoing = 0;
}
